use std::collections::HashMap;

use crate::{common::loc::LocBox, tokeniser::tokentype::*};

use super::{typ::Type, Ast};

#[derive(Debug, Clone)]
pub enum Expr {
    // Comment(Comment),
    Group(Box<LocBox<Expr>>),
    UnOp {
        typ: Punctuation,
        right: Box<LocBox<Expr>>,
    }, 
    BinOp {
        typ: Punctuation,
        left: Box<LocBox<Expr>>,
        right: Box<LocBox<Expr>>,
    },
    Literal(super::literal::Literal),
    ArrayIndex {
        name: Box<LocBox<Expr>>,
        index: Box<LocBox<Expr>>,
    },
    Path(Path),
    Call {
        path: Box<LocBox<Expr>>,
        params: CallParams, // LocBox<Expr> ~ (, Expr)*
    },
    //MethodCall {
    //    var_name: Box<LocBox<Expr>>,
    //    method_name: Ident,
    //    params: CallParams,
    //},

    /// the left side only exists on the /.|->/ chain
    FieldAccess {
        left: Box<Option<LocBox<Expr>>>,
        right: Box<LocBox<Expr>>,
    },
    PtrFieldAccess {
        left: Box<Option<LocBox<Expr>>>,
        right: Box<LocBox<Expr>>,
    },
    ForLoop {
        init: Box<Ast>,
        test: Box<LocBox<Expr>>,
        on_loop: Box<LocBox<Expr>>,
        body: Block,
    },
    WhileLoop {
        test: Box<LocBox<Expr>>,
        body: Block,
    },
    InfLoop {
        body: Block,
    },
    If(IfExpr),
    Struct {
        path: Path,
        fields: HashMap<Ident, LocBox<Expr>>,
    },
    Return(Box<Option<LocBox<Expr>>>),
    Break,
    Continue,
    Cast {
        left: Box<LocBox<Expr>>,
        right: Box<LocBox<Type>>
    },
}

impl Expr {
    pub fn unwrap_path(&self) -> Path {
        let Expr::Path(p) = self else {panic!("Unwrapping")};
        p.clone()
    }
}



#[derive(Debug, Clone)]
pub struct CallParams(pub Vec<LocBox<Expr>>);

#[derive(Debug, Clone)]
pub struct Block(pub Vec<Ast>);

#[derive(Debug, Clone)]
pub struct Path(pub Vec<Ident>);


#[derive(Debug, Clone)]
pub struct IfExpr {
    pub test: Box<LocBox<Expr>>,
    pub body: Block,
    pub else_if: Option<IfBranchExpr>
}

#[derive(Debug, Clone)]
pub enum IfBranchExpr {
    ElseIf(Box<IfExpr>),
    Else(Block)
}

