
use crate::common::{loc::LocBox, Loc};

use super::{expr::{Block, Expr}, typ::Type, Ident, TString};

#[derive(Debug, Clone)]
pub enum Statement {
    Fn(Function),
    TypeAlias(TypeAlias),
    Struct(Struct),
    Enum(Enum),
    ConstVar {
        name: Ident,
        typ: LocBox<Type>,
        val: LocBox<Expr>
    },
    StaticVar {
        name: Ident,
        typ: LocBox<Type>,
        val: LocBox<Expr>,
    },
    Let {
        name: Ident,
        typ: Option<LocBox<Type>>,
        val: Option<LocBox<Expr>>,
    },
}

#[derive(Debug, Clone)]
pub struct TypeAlias {
    pub name: Ident,
    pub typ: LocBox<Type>,
}

#[derive(Debug, Clone)]
pub struct Struct {
    pub name: Ident,
    pub fields: Vec<(Ident, LocBox<Type>)>,
}

#[derive(Debug, Clone)]
pub struct Enum {
    pub name: Ident,
    pub fields: Vec<Ident>,
}

#[derive(Debug, Clone)]
pub struct Function {
    pub struct_name: Option<Ident>,
    pub name: Ident,
    pub params: Vec<(Ident, LocBox<Type>)>,
    pub ret_type: Option<LocBox<Type>>,
    pub qual_const: bool,
    pub qual_extern: Option<TString>, // abi
    pub body: Option<Block>, // If None then its a type declaration
}
